proc _deet_symval {v} {
	global gdbInfo
	sendCommand "print (void *) & $v"
	set addr $gdbInfo(address)
	if {$addr == ""} {
		sendCommand "up 0"
		if {$gdbInfo(error) != ""} {error "Symbol not found"}
		set addr [list $gdbInfo(frame) $gdbInfo(func) $v]
	}
	return $addr
}

proc deet_sym {opt {v ""}} {
	global gdbInfo

	if {$gdbInfo(running)} {error "Program running"}
	set gdbInfo(vars) ""

	switch -regexp -- $opt {
	  {^-f.*} {
 	  # List all files - if we can't tell, it's OK to return nothing
		sendCommand "info sources"
		return $gdbInfo(files)
	  }
	  {^-n.*} {
	  # Find specific name
		set gdbInfo(type) ""
		set gdbInfo(address) ""
		set gdbInfo(file) ""
		set gdbInfo(line) 0
		sendCommand "whatis $v"
		if {$gdbInfo(type) == ""} {
			error "Symbol $v not found"
		}
		sendCommand "info line $v"
		return [list "$gdbInfo(file):$gdbInfo(line).0" \
			$gdbInfo(type) [_deet_symval $v]]
	  }
	  {^(-l|-p).*} {
	  # Print locals
		set gdbInfo(vars) ""
		set result ""
		if {[string match "-l*" $opt]} {
			sendCommand "info locals"
		} else {
			sendCommand "info args"
		}
		foreach i $gdbInfo(vars) {
			sendCommand "print &$i"
			sendCommand "whatis $i"
			lappend result $i $gdbInfo(type) $gdbInfo(address)
		}
		return $result
	  }
	}
}

proc deet_type {tp} {
	global gdbInfo

	if {$gdbInfo(running)} {error "Program running"}
	regsub -all = $tp " " name
	switch -regexp -- $tp {
		{\*$} {
			return "pointer [shpat_%% $tp {=*\*}]"
		}
		{.*\(\)$} {
			return "pointer [shpat_%% $tp {=*\(\)}]"
		}
		{\[[0-9]+\]$} {
			set atp [shpat_% $tp {.*=*\[[0-9]+\]}]
			set elm [shpat_# $tp "$atp.*\\\["]
			set elm [shpat_% $elm {\]}]
			return "array $atp $elm"
		}
		{^struct.*} {
			sendCommand "ptype $name"
			return "$name $gdbInfo(members)"
		}
		{^void$} {
			return void
		}
		{^(unsigned=)*(int|long|short|char|float|double|long=double)$} {
			return [list $name]
		}
		{.*} {
			set gdbInfo(error) ""
			sendCommand "ptype $name"
			if {$gdbInfo(error) == "NO_SYM"} {return 1}
			return "[list struct $name] $gdbInfo(members)"
		}
	}
	return 0
}

proc deet_getval {tp val {start 0} {num 10}} {
	global gdbInfo

	if {$gdbInfo(running)} {error "Program running"}
	
	set symAddress 1
	if {[llength $val] > 1} {
		set symAddress 0
		set frame [lindex $val 0]
		set name [lindex $val 1]
		set val [lindex $val 2]
		sendCommand "select-frame $frame"
	}
	regsub -all = $tp " " nm
	switch -regexp -- $tp {
	   {\[[0-9]+\]$} {
		set atp [shpat_% $nm {.* *\[[0-9]+\]}]
		set elm [shpat_% [shpat_# $nm "$atp.*\\\["] {\]}]
		if [expr $elm < $num] {
			set num $elm
		}
		set result ""
		for {set i $start} {$i < ($start + $num)} {incr i} {
			if {$symAddress} {
				sendCommand "print (($atp *) $val )+$i"
				lappend result $gdbInfo(address)
			} else {lappend result [list $frame $name "$val\[$i\]"]}
		}
		return $result
	   }
	   {(unsigned=)*(int|long|short|char|float|double|long=double)$} {
		set gdbInfo(address) ""
		if {$symAddress} {
			sendCommand "print * ( ($nm *) $val )"
		} else {
			sendCommand "print ($nm) $val"
		}
		return $gdbInfo(address)
	   }
	   {.*\*$} {
		set gdbInfo(address) ""
		if {$symAddress} {
			sendCommand "print * ( ($nm *) $val )"
		} else {
			sendCommand "print ($nm) $val"
		}
		return [string trim "$gdbInfo(address) $gdbInfo(string)"]
	   }
	   {.*\(\)$} {
		if {$symAddress} {
			return $val
		} else {
			sendCommand "print (void *) $val"
			return $gdbInfo(address)
		}
	   }
	   {(.*|struct=.*[^\*]$)} {
		set gdbInfo(members) ""
		sendCommand "ptype $nm"
		if {$gdbInfo(error) == "NO_SYM"} {return 1}
		set members $gdbInfo(members)
		set result ""
		foreach m $members {
		   set gdbInfo(address) ""
		   if {$symAddress} {
			sendCommand "print & ((($nm *) $val)->[lindex $m 1])"
			lappend result $gdbInfo(address)
		   } else {
			lappend result [list $frame $name "$val.[lindex $m 1]"]
		   }
		}
		return $result
	  }
	}
}
